/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import org.apache.log4j.Logger;

/**
 * This class is used to control the resend business process
 *
 * @author Kevin Jin
 * version 1.0 - 28 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 - Modified logging, added exceptionwaittime
 * version 1.3 - 19-May-2004 added stop file processing, log4j, move file to plinbox on resend
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * @version 1.5.1 5-Nov-2006 added partSleep to improve reponsiveness when stopping
 */

public class ResendProcess extends PbThread {

   private String instanceName = "";
   private long zipsSent = 0;
   //private volatile boolean stop = false;
   private static Logger logger = Logger.getLogger(ResendProcess.class);

   /**
    * Constructor
    *
    */
   public ResendProcess() {
      this.instanceName = Constants.getConstant().getInstanceName();
   }

   /**
    * Thread that periodically checks to see if resend is required from previously
    * rejected files.
    */
   public void run() {

      while (!getStop()) {

         long waitingTime = Constants.getConstant().getResendWaitTime();
         zipsSent = 0;

         try {

            // Start the resend processes
            Util_Logger.debug(logger, "Resend Process Started");

            // First Let's get the Resend File list
            FileManager fm = new FileManager();

            String plInboxDir = Constants.getConstant().getPLInboxDir();
            String resendDir = plInboxDir + "/resend";
            String timeoutDir = plInboxDir + "/timeout";
            String[] pliRESENDFileList = fm.scanDir(resendDir, new BatcherFileFilter(".resend"));

            // scan the list of .resend files
            if ((pliRESENDFileList != null) && (pliRESENDFileList.length > 0)) {

               Util_Logger.debug(logger, "" + pliRESENDFileList.length + " files will be processed for resend");

               // We do have .resend file
               for (int i = 0; i < pliRESENDFileList.length; i++) {
                  String pliRESENDFile = pliRESENDFileList[i];

                  if (getStop()) {
                     break;
                  }

                  Util_Logger.debug(logger, "Start to process file " + pliRESENDFile);
                  int tryNo = ResendTryController.getInstance(instanceName).getTryNumber(pliRESENDFile);

                  if (tryNo >= Constants.getConstant().getResendTryControlNo()) {
                     // copy the file to timeout directory
                     fm.copyLocalFile(resendDir, timeoutDir, pliRESENDFile);
                     Util_Logger.debug(logger, "Copied file " + pliRESENDFile + " to timeout sub-directory");
                     ResendTryController.getInstance(instanceName).deleteEntry(pliRESENDFile);
                     continue;
                  } else {
                     // rename back to .zip file
                     String pliZIPFile = fm.replaceFileName(pliRESENDFile, "zip");
                     fm.renameFile(resendDir, pliRESENDFile, pliZIPFile);

                     // update the munber
                     tryNo = tryNo + 1;
                     ResendTryController.getInstance(instanceName).updateTryNumber(pliRESENDFile, tryNo);

                     //Copy .zip to plinbox to be sent by the normal sending process
                     fm.copyLocalFile(resendDir, plInboxDir, pliZIPFile);
                     Util_Logger.debug(logger, "Copied file " + pliZIPFile + " to " + plInboxDir);
                     zipsSent++;

                     Util_Logger.debug(logger, "Finish processing file " + pliRESENDFile);
                  }
               }

               Util_Logger.debug(logger, "" + pliRESENDFileList.length + " files have been processed for resend");

            } else {
               // no file to be processed
               Util_Logger.debug(logger, "No file will be processed for resend");
            }

            // logging
            if (zipsSent > 0) {
               Util_Logger.info(logger, "End Resend " + zipsSent + " zips re sent");
            } else {
               Util_Logger.debug(logger, "End Resend " + zipsSent + " zips re sent");
            }

         } catch (Exception e) {

            // we need to set the waiting time to the value for the exception
            // so that we dont flood the log
            waitingTime = Constants.getConstant().getExceptWaitTime();
            Util_Logger.warning(logger, "Error occured when running resend process", e);

         }

         partSleep(waitingTime);

      }

   }

}
