/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.util;

/**
 * This class is a generic Exception class for all fatal errors happened
 * during this application running.
 *
 * The main idea is to keep error logID, error message, error Level (ERROR),
 * as well as stack trace information. So that system administrator can check it to
 * identify the Error causing reason.
 *
 * @author Kevin Jin
 * version 1.0 - 01 Sep 2003 - Creation
 * @version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 */

public class ApplicationException extends RuntimeException {
   //public class ApplicationException extends Exception {

   // Variables
   private String logID; // Exception log ID
   private String message; // Exception error message
   private String debugLevel; // Exception logging level
   private String stackTrace; // Exception eroor message stack trace

   /**
    * Default Constructor
    */
   public ApplicationException() {}

   /**
    * Constructor containing all information.
    *
    * @param logID String
    * @param message String
    * @param debugLevel String
    * @param stackTrace String
    */
   public ApplicationException(String logID, String message, String debugLevel, String stackTrace) {
      this.logID = logID;
      this.message = message;
      this.debugLevel = debugLevel;
      this.stackTrace = stackTrace;
   }

   /**
    * Access Method to set error log ID
    *
    * @param logID String
    */
   public void setLogID(String logID) {
      this.logID = logID;
   }

   /**
    * Access Method to get error log ID
    *
    * @return String logID
    */
   public String getLogID() {
      return this.logID;
   }

   /**
    * Access Method to set error message
    *
    * @param message String
    */
   public void setErrorMessage(String message) {
      this.message = message;
   }

   /**
    * Access Method to get error message
    *
    * @return String message
    */
   public String getErrorMessage() {
      return this.message;
   }

   /**
    * Access Method to set logging level
    *
    * @param loggingLevel String debugLevel
    */
   public void setDebugLevel(String loggingLevel) {
      this.debugLevel = loggingLevel;
   }

   /**
    * Access Method to get logging level
    *
    * @return String debugLevel
    */
   public String getDebugLevel() {
      return this.debugLevel;
   }

   /**
    * Access Method to set error stack trace information
    *
    * @param stackTrace String
    */
   public void setExceptionStackTrace(String stackTrace) {
      this.stackTrace = stackTrace;
   }

   /**
    * Access Method to get error stack trace information
    *
    * @return String stackTrace
    */
   public String getExceptionStackTrace() {
      return this.stackTrace;
   }

}
