/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7.
 * 31-Aug-2012     Bob Jones          7984 Update version number and date.
 * 10-Feb-2015     Warwick Mills      5252 Metering Batcher fails when duplicate file exists
 * 20-Jul-2016     Eddie Graham       6296 Enumeraions
 * 25-Sep-2018     Harpreet Kaur      DFCT0020271 Update version number for LifeSupport Changes
 * 06-Mar-2019     Deepak Gupta       DFCT0022009
 * 24-Feb-2020     Harpreet Kaur      Updated application version with respect to schema change for Customer Switching
 */

package com.nemmco.participantbatcher.util;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class is used to hold all informations stored in the
 * "NAME.properties" file and also define other constant data
 * for the runtime.
 *
 * author Kevin Jin
 * version 1.0   - 27-Aug-2003 Creation
 * version 1.1 4 -    Nov-2003 Added exceptWaitTime, logginglevel
 * version 1.3   - 19-May-2004 Added stopFilePath
 * version 1.3.3 - 16-Feb-2005 Added ftpPassiveMode
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.2 - 17-Dec-2005 Added custom file clean items; port
 * version 1.4.4 - 24-Apr-2006 Added FTP EDT 3rd party software, added debug routine for logging
 * version 1.5.0 - 10-Jun-2006 webDAV added, added constant changer updateConstants, made FTP passive the default
 * version 1.5.1 -  3-Nov-2006 Made the process type default to BOTH, added defaults for timers
 * version 1.5.3 - 18-Jul-2007 Make the getConstant thread safe
 * version 2.0.1 - 15-Nov-2010 Peer review work
 * version 2.0.2 - 19-Nov-2010 Peer review work, created APPLICATION_VERSION
 * version 2.0.3 - 22-Feb-2011 New version
 * version 2.0.4 - 17-Jan-2012 New version for updated aseXML schema.
 * version 2.0.6 - 17-May-2013 New version for aseXML schema update to r32.
 * version 2.0.7 - 10-Feb-2015 Overwrite duplicate files
 * version 2.0.8 - 20-Jul-2016 New version
 * version 2.0.9 - 24-Jan-2017 New version
 * version 2.0.10 -13-Jul-2017 New version
 * version 2.0.11 -25-Sep-2018 New version
 * version 2.0.13 - 21-Dec-2020 Market Element is a mandatory element for 5MS changes. This fix is for Issue 3481 - SIT3C5 - Market element is missing in the mack generated in participant inbox by participant and instance batcher.Issue 3481 - SIT3C5 - Market element is missing in the mack generated in participant inbox by participant and instance batcher.

 */


public class Constants {

   private static Logger logger = Logger.getLogger(Constants.class);

   // The only instance of this class
   private static Constants constant = null;

   // File types used for FTP process
   public static final String BINARY_FILE_TYPE = "BINARY";
   public static final String ASCII_FILE_TYPE = "ASCII";

   // Participant batcher process types
   public static final String PROCESS_SEND = "Send";
   public static final String PROCESS_RECEIVE = "Receive";
   public static final String PROCESS_BOTH = "Both";
   public static final String APPLICATION_VERSION = "v2.0.13";

   // Acknowledgement validation types
   public static final String POSITIVE_ACK = "Positive";
   public static final String NEGATIVE_ACK = "Negative";
   public static final String RESEND_ACK = "Resend";

	public static final String MARKET = "NEM";

   // FTP server information
   private String ftpServerName = "";
   private String userName = "";
   private String password = "";
   private String domain = ""; //v1.5.0 for webDAV
   private boolean ftpPassiveMode = true; //v1.5.0 make passive the default
   private int lowFtpPort = 0;
   private int highFtpPort = 0; //v1.4.4 for FtpEdt client software
	// FTP file transfer buffer size
	private int bufferSize = 0;

   public static final String COM_MODE_FTP = "FTP";
   public static final String COM_MODE_FTP_EDT = "FTPEDT";
   public static final String COM_MODE_SHARE = "Share";
   public static final String COM_MODE_WEBDAV = "webDAV";
   private String comMode = COM_MODE_FTP;

   // Business Process Type
   // Valid values are : Send, Receive and Both
   private String processType = PROCESS_BOTH; //v1.5.1

   // File Types (Character Filters) -- Should start with "filetype"
   // Note that if you do not specify file type, the application will work
   // on all file in the directory.
   private Vector<String> fileTypes = new Vector<String>();

   // B2B File Types (Character Filters)
   // Note that if you do not specify b2b file type, the application will work
   // assuming all files are NOT b2b files
   private Vector<String> b2bFileTypes = new Vector<String>();

	// Accept Transaction Types
	// Note that if you do not specify accept transaction type, the application will work
	// as per reject transaction type (if present), else it will process all the transaction types.
	private Vector<String> acceptTxTypes = new Vector<String>();
	// Reject Transaction Types
	// Note that if you do not specify reject transaction type, the application will work
	// as per accept transaction type (if present), else it will process all the transaction types.
	private Vector<String> rejectTxTypes = new Vector<String>();
	// Accept Schemas
	// Note that if you do not specify accept schemas, the application will work
	// as per reject schemas (if present), else it will process all the schemas.
	private Vector<String> acceptSchemas = new Vector<String>();
	// Reject Schemas
	// Note that if you do not specify reject schemas, the application will work
	// as per accept schemas(if present), else it will process all the schemas.
	private Vector<String> rejectSchemas = new Vector<String>();

   // Schema Validation
   // true - if schema validation should be carried out; false - otherwise
   private boolean validateSchema = true;

   // Watermarks for Participant NEMMCO Inbox
   private int inboxHighMark = 30;
   private int inboxLowMark = 10;

   // Watermarks for Participant Local Outbox
   private int outboxHighMark = 500;
   private int outboxLowMark = 50;

   // Waiting time between each file for the main process
   private int fileWaitTime = 0;

   // Waiting time between each process for the main process
   private int processWaitTime = 30000;

   // Waiting time between each process for the main process after an exception
   private int exceptWaitTime = 60000;

   // Waiting time between each resend process
   private int resendWaitTime = 300000;

   // Waiting time between each cleanup process
   private int cleanWaitTime = 30000;

   // Waiting time between each monitoring process
   private int monitorWaitTime = 30000;

   // FTP file transfer timeout
   private int timeout = 10000;

   // Force overwrite of existing local files
   private boolean fileOverwrite = false;

   // Age limit for files in custom clean archive
   private int cleanAge = 0;

   // User Directories
   // The data should include participant NEMMCO inbox, participant NEMMCO outbox, participant NEMMCO stopbox
   //                         participant local inbox, participant local outbox
   private String pnInbox = "";
   private String pnOutbox = "";
   private String pnStopbox = "";
   private String plInbox = "";
   private String plOutbox = "";

   // StopFilePath
   private String stopFileDir = "";

   // stop file wait time in millsec
   private int stopFileWaitTime = 35000;

   // Monitor File Directory
   private String monitorFileDir = "";

   // Resend Try Control File Directory & Numner
   private String resendTryControlFileDir = "";
   private int resendTryControlNo = 0;

   // Acknowledgement file header
   private String ackSchemaNamespace = "";
   private String ackSchemaLocation = "";

   // Schema Caching Locations
   private Vector<String> schemaCaches = new Vector<String>();

   // Custom Clean definitions
   private Vector<CustomClean> customClean = new Vector<CustomClean>();

   private String instanceName = "";

   // Port for HttpServer
   private int port = 0;

   // Constructor
   // This class follows singleton patten, so the constructor is set to private
   private Constants() {}

   /**
    * This class is used to set the only copy of this Constants object.
    */
   private static class SingletonConstantsHolder {

      private static Constants instance = new Constants();
   }

   /**
    * This method is used to get the only copy of this Constants object.
    *
    * Note: this class follows singletion pattern, so the constructor is
    * defined as private. users can get the only Constants object from
    * this method.
    *
    * @return Constants - the only copy of the Constants object
    */
   public static Constants getConstant() {
      if (constant == null) {
         constant = SingletonConstantsHolder.instance;
      }
      return constant;
   }

   // Getters and Setters

   /**
    * Access method to set ftpServerName
    *
    * @param ftpServerName String
    */
   public void setFTPServerName(String ftpServerName) {
      this.ftpServerName = ftpServerName;
   }

   /**
    * Access method to get ftpServerName
    *
    * @return ftpServerName String
    */
   public String getFTPServerName() {
      return this.ftpServerName;
   }

   /**
    * Access method to set userName
    *
    * @param userName String
    */
   public void setUserName(String userName) {
      this.userName = userName;
   }

   /**
    * Access method to get userName
    *
    * @return userName String
    */
   public String getUserName() {
      return this.userName;
   }

   /**
    * Access method to set password
    *
    * @param password String
    */
   public void setPassword(String password) {
      this.password = password;
   }

   /**
    * Access method to get password
    *
    * @return password String
    */
   public String getPassword() {
      return this.password;
   }

   /**
    * Access method to set domain
    *
    * @param domain String
    */
   public void setDomain(String domain) {
      this.domain = domain;
   }

   /**
    * Access method to get domain
    *
    * @return domain String
    */
   public String getDomain() {
      return this.domain;
   }

   /**
    * Access method to set Ftp Mode
    *
    * @param mode boolean set to true for Passive mode
    */
   public void setFtpPassiveMode(boolean mode) {
      this.ftpPassiveMode = mode;
   }

   /**
    * Access method to get Ftp Passive Mode
    *
    * @return mode boolean
    */
   public boolean getFtpPassiveMode() {
      return this.ftpPassiveMode;
   }

   /**
    * Access method to set com mode
    *
    * @param mode set the communication mode to FTP, FTPEDT or SHARE
    */
   public void setComMode(String mode) {
      this.comMode = mode;
   }

   /**
    * Access method to get com mode
    *
    * @return com mode - String
    */
   public String getComMode() {
      return this.comMode;
   }

   /**
    * Access method to set process type
    *
    * @param processType String
    */
   public void setProcessType(String processType) {
      this.processType = processType;
   }

   /**
    * Access method to get process type
    *
    * @return processType String
    */
   public String getProcessType() {
      return this.processType;
   }

   /**
    * Access method to set Low Ftp Port
    *
    * @param lowFtpPort int
    */
   public void setLowFtpPort(int lowFtpPort) {
      this.lowFtpPort = lowFtpPort;
   }

   /**
    * Access method to get Low FFtp Port
    *
    * @return lowFtpPort int
    */
   public int getLowFtpPort() {
      return this.lowFtpPort;
   }

   /**
    * Access method to set High Ftp Port
    *
    * @param highFtpPort int
    */
   public void setHighFtpPort(int highFtpPort) {
      this.highFtpPort = highFtpPort;
   }

   /**
    * Access method to get High Ftp Port
    *
    * @return highFtpPort int
    */
   public int getHighFtpPort() {
      return this.highFtpPort;
   }

	/**
	* Access method to set FTP Buffer Size
	* @param bufferSize int FTP buffer size in bytes
	*/
	public void setBufferSize(int bufferSize) {
		this.bufferSize = bufferSize;
	}

	/**
	* Access method to get FTP Buffer Size
	* @return bufferSize int
	*/
	public int getBufferSize() {
		return this.bufferSize;
	}

   /**
    * Access method to add file type
    *
    * @param fileType String
    */
   public void addFileType(String fileType) {
      this.fileTypes.add(fileType);
   }

   /**
    * File masks of all files to be processed by this instance
    * @param paramVector Vector contains the list for file masks
    */
   public void setFileType(Vector<String> paramVector) {
      this.fileTypes = paramVector;
   }

   /**
    * Access method to get file types
    *
    * @return Vector fileTypes
    */
   public Vector<String> getFileTypes() {
      return this.fileTypes;
   }

   /**
    * Access method to add file type
    *
    * @param b2bFileType add a file type file mask to identify B2B files
    */
   public void addB2BFileType(String b2bFileType) {
      this.b2bFileTypes.add(b2bFileType);
   }

   /**
    * Set the file masks for B2B files
    * @param paramVector list of file masks
    */
   public void setB2BFileType(Vector<String> paramVector) {
      this.b2bFileTypes = paramVector;
   }

   /**
    * Access method to get file types
    *
    * @return fileTypes Vector
    */
   public Vector<String> getB2BFileTypes() {
      return this.b2bFileTypes;
   }

	/**
	 * Access method to add a transaction type to be accepted
	 *
	 * @param acceptTxType add a transaction type to be processed
	 */
	public void addAcceptTxType(String acceptTxType) {
		this.acceptTxTypes.add(acceptTxType);
	}

	/**
	 * Set the transaction types to be processed
	 * @param paramVector list of accepted transaction types
	 */
	public void setAcceptTxTypes(Vector<String> acceptTxTypes) {
		this.acceptTxTypes = acceptTxTypes;
	}

	/**
	 * Access method to get acceptTxTypes
	 *
	 * @return acceptTxTypes Vector
	 */
	public Vector<String> getAcceptTxTypes() {
		return this.acceptTxTypes;
	}

	/**
	 * Access method to add a transaction type to be ignored
	 *
	 * @param rejectTxType add a transaction type to be ignored
	 */
	public void addRejectTxType(String rejectTxType) {
		this.rejectTxTypes.add(rejectTxType);
	}

	/**
	 * Set the transaction types to be ignored
	 * @param paramVector list of rejected transaction types
	 */
	public void setRejectTxTypes(Vector<String> rejectTxTypes) {
		this.rejectTxTypes = rejectTxTypes;
	}

	/**
	 * Access method to get rejectTxTypes
	 *
	 * @return rejectTxTypes Vector
	 */
	public Vector<String> getRejectTxTypes() {
		return this.rejectTxTypes;
	}

	/**
	 * Access method to add a schema to be accepted
	 *
	 * @param acceptSchema add a schema to be processed
	 */
	public void addAcceptSchema(String acceptSchema) {
		this.acceptSchemas.add(acceptSchema);
	}

	/**
	 * Set the schema types to be processed
	 * @param paramVector list of accepted schemas
	 */
	public void setAcceptSchemas(Vector<String> acceptSchemas) {
		this.acceptSchemas = acceptSchemas;
	}

	/**
	 * Access method to get acceptSchemas
	 *
	 * @return acceptSchemas Vector
	 */
	public Vector<String> getAcceptSchemas() {
		return this.acceptSchemas;
	}

	/**
	 * Access method to add a schema to be ignored
	 *
	 * @param rejectSchema add a schema to be ignored
	 */
	public void addRejectSchema(String rejectSchema) {
		this.rejectSchemas.add(rejectSchema);
	}

	/**
	 * Set the schema types to be ignored
	 * @param paramVector list of rejected Schemas
	 */
	public void setRejectSchemas(Vector<String> rejectSchemas) {
		this.rejectSchemas = rejectSchemas;
	}

	/**
	 * Access method to get rejectSchemas
	 *
	 * @return rejectSchemas Vector
	 */
	public Vector<String> getRejectSchemas() {
		return this.rejectSchemas;
	}

   /**
    * Access method to set schema validation
    *
    * @param validateSchema set to true if you want files to be validated against the schema - normally true
    */
   public void setValidateSchema(boolean validateSchema) {
      this.validateSchema = validateSchema;
   }

   /**
    * Access method to check schema validation
    *
    * @return processType boolean
    */
   public boolean isValidateSchema() {
      return this.validateSchema;
   }

   /**
    * Access method to set Participant NEMMCO Inbox high watermark
    *
    * @param inboxHighMark int
    */
   public void setInboxHighMark(int inboxHighMark) {
      this.inboxHighMark = inboxHighMark;
   }

   /**
    * Access method to get Participant NEMMCO Inbox high watermark
    *
    * @return inboxHighMark int
    */
   public int getInboxHighMark() {
      return this.inboxHighMark;
   }

   /**
    * Access method to set Participant NEMMCO Inbox low watermark
    *
    * @param inboxLowMark int
    */
   public void setInboxLowMark(int inboxLowMark) {
      this.inboxLowMark = inboxLowMark;
   }

   /**
    * Access method to get Participant NEMMCO Inbox low watermark
    *
    * @return inboxLowMark int
    */
   public int getInboxLowMark() {
      return this.inboxLowMark;
   }

   /**
    * Access method to set Participant Local Outbox high watermark
    *
    * @param outboxHighMark int
    */
   public void setOutboxHighMark(int outboxHighMark) {
      this.outboxHighMark = outboxHighMark;
   }

   /**
    * Access method to get Participant Local Outbox high watermark
    *
    * @return outboxHighMark int
    */
   public int getOutboxHighMark() {
      return this.outboxHighMark;
   }

   /**
    * Access method to set Participant Local outbox low watermark
    *
    * @param outboxLowMark int
    */
   public void setOutboxLowMark(int outboxLowMark) {
      this.outboxLowMark = outboxLowMark;
   }

   /**
    * Access method to get Participant Local Outbox low watermark
    *
    * @return outboxLowMark int
    */
   public int getOutboxLowMark() {
      return this.outboxLowMark;
   }

   /**
    * Access method to set File Waiting Time
    *
    * @param fileWaitTime int
    */
   public void setFileWaitTime(int fileWaitTime) {
      this.fileWaitTime = fileWaitTime;
   }

   /**
    * Access method to get File Waiting Time
    *
    * @return fileWaitTime int
    */
   public int getFileWaitTime() {
      return this.fileWaitTime;
   }

   /**
    * Access method to set Process Waiting Time
    *
    * @param processWaitTime int
    */
   public void setProcessWaitTime(int processWaitTime) {
      this.processWaitTime = processWaitTime;
   }

   /**
    * Access method to get Process Waiting Time
    *
    * @return processWaitTime int
    */
   public int getProcessWaitTime() {
      return this.processWaitTime;
   }

   /**
    * Access method to set Except Waiting Time
    *
    * @param exceptWaitTime int
    */
   public void setExceptWaitTime(int exceptWaitTime) {
      this.exceptWaitTime = exceptWaitTime;
   }

   /**
    * Access method to get Except Waiting Time
    *
    * @return exceptWaitTime int
    */
   public int getExceptWaitTime() {
      return this.exceptWaitTime;
   }

   /**
    * Access method to set Resend Waiting Time
    *
    * @param resendWaitTime int
    */
   public void setResendWaitTime(int resendWaitTime) {
      this.resendWaitTime = resendWaitTime;
   }

   /**
    * Access method to get Resend Waiting Time
    *
    * @return resendWaitTime int
    */
   public int getResendWaitTime() {
      return this.resendWaitTime;
   }

   /**
    * Access method to set Cleanup Waiting Time
    *
    * @param cleanWaitTime int
    */
   public void setCleanupWaitTime(int cleanWaitTime) {
      this.cleanWaitTime = cleanWaitTime;
   }

   /**
    * Access method to get Cleanup Waiting Time
    *
    * @return cleanWaitTime int
    */
   public int getCleanWaitTime() {
      return this.cleanWaitTime;
   }

   /**
    * Access method to set Monitor Waiting Time
    *
    * @param monitorWaitTime int
    */
   public void setMonitorWaitTime(int monitorWaitTime) {
      this.monitorWaitTime = monitorWaitTime;
   }

   /**
    * Access method to get Monitor Waiting Time
    *
    * @return monitorWaitTime int
    */
   public int getMonitorWaitTime() {
      return this.monitorWaitTime;
   }

   /**
    * Access method to set FTP Time Out
    *
    * @param timeout int millisec FTP timeout for file operations etc
    */
   public void setTimeout(int timeout) {
      this.timeout = timeout;
   }

   /**
    * Access method to get FTP Time Out
    *
    * @return timeout int
    */
   public int getTimeout() {
      return this.timeout;
   }

   /**
    * Access method to set existing file overwrite
    *
    * @param fileOverwrite control overwrite of existing files
    */
   public void setFileOverwrite(boolean fileOverwrite) {
      this.fileOverwrite = fileOverwrite;
   }

   /**
    * Access method to get existing file overwrite
    *
    * @return fileOverwrite boolean
    */
   public boolean getFileOverwrite() {
      return this.fileOverwrite;
   }

   /**
    * Access method to set age limit for files in custom clean archive
    *
    * @param cleanAge age limit
    */
   public void setCleanAge(int cleanAge) {
      this.cleanAge = cleanAge;
   }

   /**
    * Access method to get age limit for files in custom clean archive
    *
    * @return cleanAge int
    */
   public int getCleanAge() {
      return this.cleanAge;
   }

   /**
    * Access method to set Participant NEMMCO Inbox directory
    *
    * @param pnInbox String
    */
   public void setPNInboxDir(String pnInbox) {
      this.pnInbox = pnInbox;
   }

   /**
    * Access method to get Participant NEMMCO Inbox directory
    *
    * @return pnInbox String
    */
   public String getPNInboxDir() {
      return this.pnInbox;
   }

   /**
    * Access method to set Participant NEMMCO Outbox directory
    *
    * @param pnOutbox String
    */
   public void setPNOutboxDir(String pnOutbox) {
      this.pnOutbox = pnOutbox;
   }

   /**
    * Access method to get Participant NEMMCO Outbox directory
    *
    * @return pnOutbox String
    */
   public String getPNOutboxDir() {
      return this.pnOutbox;
   }

   /**
    * Access method to set Participant NEMMCO Outbox directory
    *
    * @param pnStopbox String
    */
   public void setPNStopboxDir(String pnStopbox) {
      this.pnStopbox = pnStopbox;
   }

   /**
    * Access method to get Participant NEMMCO Outbox directory
    *
    * @return pnStopbox String
    */
   public String getPNStopboxDir() {
      return this.pnStopbox;
   }

   /**
    * Access method to set Participant Local Inbox directory
    *
    * @param plInbox String
    */
   public void setPLInboxDir(String plInbox) {
      this.plInbox = plInbox;
   }

   /**
    * Access method to get Participant Local Inbox directory
    *
    * @return plInbox String
    */
   public String getPLInboxDir() {
      return this.plInbox;
   }

   /**
    * Access method to set Participant Local Outbox directory
    *
    * @param plOutbox String
    */
   public void setPLOutboxDir(String plOutbox) {
      this.plOutbox = plOutbox;
   }

   /**
    * Access method to get Participant Local Outbox directory
    *
    * @return plOutbox String
    */
   public String getPLOutboxDir() {
      return this.plOutbox;
   }

   /**
    * Access method to set stopFile path
    *
    * @param stopFileDir the directory where the stop file is expected to be lodged
    */
   public void setStopFileDir(String stopFileDir) {
      this.stopFileDir = stopFileDir;
   }

   /**
    * Access method to get stopFile path
    *
    * @return stopFilePath String
    */
   public String getStopFileDir() {
      return this.stopFileDir;
   }

   /**
    * Access method to set Log file directory
    *
    * @param stopFileWaitTime time to wait in millisec
    * when stopping the process - set to be about the same time
    * of the longest cycle time that you dont wish to interrupt
    */
   public void setStopFileWaitTime(int stopFileWaitTime) {
      this.stopFileWaitTime = stopFileWaitTime;
   }

   /**
    * Access method to get Log file directory
    *
    * @return stopFileWait int
    */
   public int getStopFileWaitTime() {
      return this.stopFileWaitTime;
   }

   /**
    * Access method to set Monitor file directory
    *
    * @param monitorFileDir String
    */
   public void setMonitorFileDir(String monitorFileDir) {
      this.monitorFileDir = monitorFileDir;
   }

   /**
    * Access method to get Log file directory
    *
    * @return monitorFileDir String
    */
   public String getMonitorFileDir() {
      return this.monitorFileDir;
   }

   /**
    * Access method to set Resend Try Control file directory
    *
    * @param resendTryControlFileDir String
    */
   public void setResendTryControlFileDir(String resendTryControlFileDir) {
      this.resendTryControlFileDir = resendTryControlFileDir;
   }

   /**
    * Access method to get Resend Try Control file directory
    *
    * @return resendTryControlFileDir String
    */
   public String getResendTryControlFileDir() {
      return this.resendTryControlFileDir;
   }

   /**
    * Access method to set Resend Try Control Number
    *
    * @param resendTryControlNo int
    */
   public void setResendTryControlNo(int resendTryControlNo) {
      this.resendTryControlNo = resendTryControlNo;
   }

   /**
    * Access method to get Resend Try Control Number
    *
    * @return resendTryControlNo int
    */
   public int getResendTryControlNo() {
      return this.resendTryControlNo;
   }

   /**
    * Access method to set Ack Schema Namespace
    *
    * @param ackSchemaNamespace String
    */
   public void setAckSchemaNamespace(String ackSchemaNamespace) {
      this.ackSchemaNamespace = ackSchemaNamespace;
   }

   /**
    * Access method to get Ack Schema Namespace
    *
    * @return ackSchemaNamespace String
    */
   public String getAckSchemaNamespace() {
      return this.ackSchemaNamespace;
   }

   /**
    * Access method to set Ack Schema Location
    *
    * @param ackSchemaLocation String
    */
   public void setAckSchemaLocation(String ackSchemaLocation) {
      this.ackSchemaLocation = ackSchemaLocation;
   }

   /**
    * Access method to get Ack Schema Location
    *
    * @return ackSchemaLocation String
    */
   public String getAckSchemaLocation() {
      return this.ackSchemaLocation;
   }

   /**
    * Access method to add schema cache
    *
    * @param schemaCache String
    */
   public void addSchemaCache(String schemaCache) {
      this.schemaCaches.add(schemaCache);
   }

   /**
    * Access method to get schema caches
    *
    * @return schemaCaches Vector
    */
   public Vector<String> getSchemaCaches() {
      return this.schemaCaches;
   }

   /**
    * Access method to add custom clean
    *
    * @param customClean the object contains the custom claen function to add
    */
   public void addCustomClean(CustomClean customClean) {
      this.customClean.add(customClean);
   }

   /**
    * Access method to get custom clean
    *
    * @return customClean Vector
    */
   public Vector<CustomClean> getCustomClean() {
      return this.customClean;
   }

   /**
    * Access method to set Port
    *
    * @param port int
    */
   public void setPort(int port) {
      this.port = port;
   }

   /**
    * Access method to get Port
    *
    * @return port int
    */
   public int getPort() {
      return this.port;
   }

   /**
    * Access method to set Instance Name
    *
    * @param instanceName String
    */
   public void setInstanceName(String instanceName) {
      this.instanceName = instanceName;
   }

   /**
    * Access method to get Instance Name
    *
    * @return instanceName String
    */
   public String getInstanceName() {
      return this.instanceName;
   }

   /** debugConstants will list on the log file in DEBUG mode all the constants defined */
   public void debugConstants() {

      try {

         // publish the contents of the properties file as being used by this instance
         Method[] meth;
         meth = Constants.class.getMethods();

         // scan all the methods of the Constants class
         for (int i = 0; i < meth.length; i++) {
            String methodName;
            methodName = meth[i].getName();

            // for all the getters excluding some
            if (methodName.startsWith("get")) {
               if ((!methodName.equals("getConstant")) && (!methodName.equals("getPassword")) && (!methodName.equals("getClass"))) {

                  // determine thhe value returned from the getter
                  Object oo = meth[i].invoke(constant);

                  // for vectors dont display if empty
                  boolean skip = false;
                  if (oo.getClass().getName().equalsIgnoreCase("java.util.Vector")) {
                     Vector<Object> vv = (Vector<Object>)oo;
                     if (vv.isEmpty()) {
                        skip = true;
                     }
                  }
                  if (!skip) {
                     Util_Logger.debug(logger, "Property " + methodName.substring(3) + " = " + oo.toString());
                  }
               }
            }
         }

      } catch (Exception e) {
         Util_Logger.warning(logger, "debugConstants exception", e);
      }

   }

   /** updateConstants will search and find %participant% in constants
    * and replace with the string supplied v1.5.0
    */
   public int updateConstants(String participantID) {

      int Count = 0;
      try {

         // search for the string fields in this class
         Field[] field;
         field = Constants.class.getDeclaredFields();

         // scan all the fields of the Constants class
         for (int i = 0; i < field.length; i++) {

            // for all the fields
            if (field[i].getType().getName().equals("java.lang.String")) {

               String oldValue = (String)field[i].get(this);

               // %participant% is the substitution key
               if (oldValue.indexOf("%participant%") >= 0) {
                  String newValue = oldValue.replaceAll("%participant%", participantID);
                  field[i].set(this, newValue);
                  Count = Count + 1;
               }
            } else if (field[i].getType().getName().equals("java.util.Vector")) {

               Vector<CustomClean> vect = (Vector<CustomClean>)field[i].get(this);

               // do special work for the customClean vector
               if (vect == this.customClean) {

                  // for each instance of CustomClean
                  for (int j = 0; j < vect.size(); j++) {

                     // get the data and replace the string
                     CustomClean cc = vect.elementAt(j);
                     if (cc.getFromDir().indexOf("%participant%") >= 0) {
                        cc.setFromDir(cc.getFromDir().replaceAll("%participant%", participantID));
                        Count = Count + 1;
                     }
                     if (cc.getToDir().indexOf("%participant%") >= 0) {
                        cc.setToDir(cc.getToDir().replaceAll("%participant%", participantID));
                        Count = Count + 1;
                     }
                  }

                  // put it back
                  field[i].set(this, vect);
               }
            }
         }

      } catch (Exception e) {
         Util_Logger.warning(logger, "updateConstants exception", e);
      }
      return Count;

   }

}
