/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import java.lang.reflect.Method;

import java.net.Socket;

import java.util.Vector;

import org.apache.log4j.Logger;

/**  This routine initialises and is called to respond to a web page request to PB.
 *
 * @author Mike Muir
 * version 1.4.2 Created on 12 January 2006, 22:24
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards, made no extra thread,
 * version 2.0.1 15-Nov-2010 Removed Nemmco from screen, peer review work
 * @version 2.0.2 19-Nov-2010 Peer review work (inner class)
 *
 */
public class HttpClient {

   private static int count = 110; // characters per line roughly
   private static Logger logger = Logger.getLogger(HttpClient.class);

   // socket storage saved when created
   private HttpDataStore hds;
   private Constants constants;

   /**
    * Creates a new instance of HttpClient. Connects to HttpDataStore and Constants.
    */
   HttpClient() {
      hds = HttpDataStore.getHttpDataStore(); // java 1.6
      constants = Constants.getConstant(); // java 1.6
   }

   /** work thread for one screen refresh for one client.
    */
   public void screenRefresh(Socket client) {

      try {

         // establish the input and output to client browser
         BufferedReader in = new BufferedReader(new InputStreamReader(client.getInputStream()));
         PrintWriter out = new PrintWriter(new OutputStreamWriter(client.getOutputStream()));

         // respond to GET reading all input and discarding
         out.print("HTTP/1.0 200\r\nContent-Type: text/html\r\n\r\n");
         String line;
         String getLine = "";

         //GET /page.html HTTP/1.1
         //<a href=\"0.html\">Link</a>
         while ((line = in.readLine()) != null) {
            if (line.length() == 0) {
               break;
            }
            if (line.startsWith("GET /")) {
               getLine = line;
            }
         }

         // respond with html titles etc
         out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"<html><head>");
         out.println("<META HTTP-EQUIV=\"Refresh\"Content=\"30;URL=\">");
         out.println("<title>Participant Batcher (" + constants.getInstanceName() + ")</title>");
         out.println("</head>");
         out.println("<body>");
            out.println("<style type=\"text/css\"><!-- span.r {color:red} span.b {color:blue} " +
                        "THEAD {color:blue} TBODY {color:red} " +
                        "TD {color:red} TH {color:blue} "+
                        "TH.caption {color:green;font-weight:bold} TD.text {color:blue} --></style>");

         // Put the HyperLinks on top of all displays
         out.println("<a href=\"index.html\">Home</a>   ");
         out.println("<a href=\"0.html\">Files</a>   ");
         out.println("<a href=\"1.html\">Config</a>   ");
         out.println("<a href=\"2.html\">Stop</a>   ");
         out.println("<a href=\"3.html\"></a>   ");

         // choose the display from data in the GET
         switch (getLine.charAt(5)) {
         case '0':
            HttpClientFiles(out);
            break;
         case '1':
            HttpClientConfig(out);
            break;
         case '2':
            HttpClientStop(out);
            break;
         default: // home
            // status display is most useful so default
            HttpClientStatus(out);
            break;
         }

         // finish the html
         out.println("</body>");
         out.println("</html>");

         // close and terminate thread
         out.close();
         in.close();
         client.close();

      } catch (Exception e) {
         Util_Logger.warning(logger, "HttpServer HttpClient exception", e);
      }

   }

   /*
     * output the list of files in the NEMMCO in/out boxes
     *
     */
   private void HttpClientFiles(PrintWriter out) {

      int inLength = 0;
      int outLength = 0;
      if (hds.getPniFileList() != null) {
         inLength = hds.getPniFileList().length;
      }
      if (hds.getPnoFileList() != null) {
         outLength = hds.getPnoFileList().length;
      }
      out.println("<br><table border=2><caption align=top>Msats Interface</caption>" + "<thead>");
      out.println("<tr><th>InBox(" + inLength + ")" + "<th>OutBox(" + outLength + ")");
      out.println("<tbody>");

      for (int i = 0; i < Math.max(inLength, outLength); i++) {
         if (i < inLength) {
            out.println("<tr><td>" + hds.getPniFileList()[i] + " ");
         } else {
            out.println("<tr><td> ");
         }
         if (i < outLength) {
            out.println("<td>" + hds.getPnoFileList()[i] + " ");
         } else {
            out.println("<td> ");
         }
      }
      out.println("</tbody></table>");
   }

   private void HttpClientStop(PrintWriter out) {
      Constants c = Constants.getConstant();
      String sn = c.getStopFileDir() + "/" + c.getInstanceName() + ".stp";
      File af = new File(sn);
      String msg = "<br><br><br>Stop file could not be created";
      try {
         if (!af.exists()) {
            if (!af.createNewFile())
               throw new Exception("Create of stop file failed " + sn);
            FileWriter fw = new FileWriter(af.getAbsolutePath(), false);
            try {
               fw.write(sn);
            } finally {
               try {
                  fw.flush();
               } finally {
                  fw.close();
               }
            }
            msg = "<br><br><br>Stop file " + sn + " created";
         } else {
            msg = "<br><br><br>Stop file " + sn + " exists";
         }
      } catch (Exception e) {
         msg = "<br><br><br>Stop file exception " + e.getMessage();
      }
      out.println(msg);
   }
   /*
     * Print out the configuration as read into the Constants package
     * from the properties file.
     *
     */
   private void HttpClientConfig(PrintWriter out) {

      try {

         // publish the contents of the properties file as being used by this instance
         Method[] meth;
         meth = Constants.class.getMethods();
         String httpLine = "<br><span class=\"caption\">Instance Properties</span><br>";

         // scan all the methods of the Constants class
         for (int i = 0; i < meth.length; i++) {
            String methodName;
            methodName = meth[i].getName();

            // for all the getters excluding some
            if (methodName.startsWith("get")) {
               if ((!methodName.equals("getConstant")) && (!methodName.equals("getPassword")) && (!methodName.equals("getClass"))) {

                  // determine thhe value returned from the getter
                  Object constant = Constants.getConstant();
                  Object oo = meth[i].invoke(constant);

                  // for vectors dont display if empty
                  boolean skip = false;
                  if (oo.getClass().getName().equalsIgnoreCase("java.util.Vector")) {
                     Vector<Object> vv = (Vector<Object>)oo;
                     if (vv.isEmpty()) {
                        skip = true;
                     }
                  }
                  if (!skip) {
                     httpLine = httpLine + "<span class=\"b\">" + methodName.substring(3) + "=</span><span class=\"r\">" + oo.toString() + "</span> ";

                     // take a new line break after some data
                     if (httpLine.length() >= count) {
                        out.println(httpLine);
                        httpLine = "<br>";
                     }
                  }
               }
            }
         }

         // finish off the last of the data
         if (httpLine.length() > 4) {
            out.println(httpLine);
         }

      } catch (Exception e) {
         Util_Logger.warning(logger, "HttpClientConfig exception", e);
      }
   }
   /*
     * Print out the status screen
     */
   private void HttpClientStatus(PrintWriter out) {

      try {

         // Status Table
         out.println("<br><table border=2 width=400><thead>");
            out.println("<tr><th class=\"caption\">Status<th>Item<tbody>"+
                "<tr><th>Update Time<td align=right>"+hds.getUpdateDateTime()+
                "<tr><th>Start Time<td align=right>"+hds.getStartDateTime()+
                "<tr><th>Connected<td align=right>"+hds.getConnected()+
                "<tr><th>Free memory<td align=right>"+Runtime.getRuntime().freeMemory()/1024/1024+" MB");
         out.println("</tbody></table>");

         // File Counts
         HttpClient.FileTypes ftIn = new HttpClient.FileTypes(hds.getPniFileList());
         HttpClient.FileTypes ftOut = new HttpClient.FileTypes(hds.getPnoFileList());

         // file type counts
         out.println("<br><table border=2 width=400><thead>");
            out.println("<tr><th class=\"caption\">File Type Count<th>Inbox<th>Outbox<th>Local In<th>Local Out<tbody>"+
                "<tr><th>Zips<td align=right>"+ftIn.countZip+"<td align=right>"+ftOut.countZip+"<td align=right>"+
                                                    hds.getPliZipFileCount()+"<td align=right>"+hds.getPloZipFileCount()+
                "<tr><th>Acks<td align=right>"+ftIn.countAck+"<td align=right>"+ftOut.countAck+"<td align=right>"+
                                                    hds.getPliAckFileCount()+"<td align=right>"+hds.getPloAckFileCount()+
                "<tr><th>Ac1s<td align=right>"+ftIn.countAc1+"<td align=right>"+ftOut.countAc1+"<td><td>"+
                "<tr><th>Tmps<td align=right>"+ftIn.countTmp+"<td align=right>"+ftOut.countTmp+"<td><td>"+
                "<tr><th>Stps<td align=right>"+ftIn.countStp+"<td align=right>"+ftOut.countStp+"<td><td>");
         out.println("</tbody></table>");

         // File Transport Statistics
         out.println("<br><table border=2 width=400><thead>");
            out.println("<tr><th class=\"caption\">File Transport<th>Current<th>Files Last Hour<tbody>"+
                "<tr><th>Zips Sent<td align=right>"+hds.getZipsSent()+"<td align=right>"+hds.getZipsSentLastHour()+
                "<tr><th>Acks Recieved<td align=right>"+hds.getAcksRecieved()+"<td>"+
                "<tr><th>Zips Recieved<td align=right>"+hds.getZipsRecieved()+"<td align=right>"+hds.getZipsRecievedLastHour()+
                "<tr><th>Acks Sent<td align=right>"+hds.getAcksSent()+"<td>");
         out.println("</tbody></table>");

         // File Transport Daily Statistics
         out.println("<br><table border=2><thead>");
         out.println("<tr><th class=\"caption\">Files Per Hour");
         for (int i = 0; i < 24; i++) {
            out.println("<th>" + i);
         }
         out.println("<tbody><tr><th>Zips Sent");
         for (int i = 0; i < 24; i++) {
            out.println("<td align=right>" + hds.getZipsSentPerHour(i));
         }
         out.println("<tr><th>Zips Recieved");
         for (int i = 0; i < 24; i++) {
            out.println("<td align=right>" + hds.getZipsRecievedPerHour(i));
         }
         out.println("</tbody></table>");

      } catch (Exception e) {
         Util_Logger.warning(logger, "HttpClientConfig exception", e);
      }
   }

   // calculate the file types at the NEMMCO end only

   private static class FileTypes {

      // where the results are stored
      private int countZip;
      private int countAck;
      private int countAc1;
      private int countTmp;
      private int countStp;

      // create and count
      private FileTypes(String[] fileList) {
         countZip = 0;
         countAck = 0;
         countAc1 = 0;
         countTmp = 0;
         countStp = 0;
         if (fileList != null) {
            for (int i = 0; i < fileList.length; i++) {
               if ((fileList[i]).endsWith(".zip")) {
                  countZip++;
               } else {
                  if ((fileList[i]).endsWith(".ack")) {
                     countAck++;
                  } else {
                     if ((fileList[i]).endsWith(".ac1")) {
                        countAc1++;
                     } else {
                        if ((fileList[i]).endsWith(".tmp")) {
                           countTmp++;
                        } else {
                           if ((fileList[i]).endsWith(".stp")) {
                              countStp++;
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }
}

