/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.util;

import java.io.File;
import java.io.FileWriter;

import java.util.Calendar;

import java.text.SimpleDateFormat;

/**
 * This class is used to log an entry in the monitor log file,
 * if all other threads are still alive, and connection to NEMMCO exists.
 *
 * @author Kevin Jin
 * version 1.0 - 29 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 - Mike Muir - Stop the monitor file from growing
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 *
 */

public class MonitorLogger {

   /**
    * Constructor
    */
   public MonitorLogger() {}

   /**
    * This method is used to log to the monitoring control file.
    *
    * @param monitorFile File
    */
   public static void log(File monitorFile) {

      String logID = "";
      String logRecord = "";

      SimpleDateFormat dateFormat;
      dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      logID = dateFormat.format((Calendar.getInstance()).getTime());

      logRecord = "" + logID + " -- " + "All threads are alive\n";

      FileWriter fw;

      try {

         if (!monitorFile.exists()) {
            if (!monitorFile.createNewFile())
               throw new Exception("Creation of monitor log file failed " + monitorFile.getName());
         }
         // make sure the file does not grow - just need time updated
         // as the log has the sequential record as well
         fw = new FileWriter(monitorFile.getAbsolutePath(), false);
         fw.write(logRecord.toCharArray());

         fw.flush();
         fw.close();

      } catch (Exception e) {
         ; // if there is exception occured, we simply discard it
         // the external mechanism will pick it up
      }
   }

}
