/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.xml;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/** Error Handler used in BatcherDOMParser.
 * @version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 */
public class BatcherErrorHandler implements ErrorHandler {

   private String errorMessage = "";

   /**
    * Constructor
    */
   public BatcherErrorHandler() {}

   /**
    * This is the error handling method for Warning.
    *
    * @param ex the SAXpareException
    */
   public void warning(SAXParseException ex) throws SAXException {
      printError("Warning", ex);
   }

   /**
    * This is the error handling method for Error.
    *
    * @param ex the SAXpareException
    */
   public void error(SAXParseException ex) throws SAXException {
      printError("Error", ex);
   }

   /**
    * This is the error handling method for Fatal Error.
    *
    * @param ex the SAXpareException
    */
   public void fatalError(SAXParseException ex) throws SAXException {
      printError("Fatal Error", ex);
      throw ex;
   }

   /**
    * This method is used to get the error message.
    *
    * @return errorMessage String
    */
   public String getErrorMessages() {
      return this.errorMessage;
   }

   /**
    * Get the parser returned error message.
    */
   protected void printError(String type, SAXParseException ex) {

      String systemId = ex.getSystemId();
      if (systemId != null) {
         int index = systemId.lastIndexOf('/');
         if (index != -1)
            systemId = systemId.substring(index + 1);
      } else {
         systemId = "";
      }

      // Get the error message returned by the underline parser.
      if (errorMessage.length() > 0) {
         errorMessage = errorMessage + "\n";
      }
      errorMessage = errorMessage + "[" + type + "]" + systemId + ":" + ex.getLineNumber() + ":" + ex.getColumnNumber() + ":" + ex.getMessage() + ":";
   }

}
