/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.xml;

import com.nemmco.participantbatcher.process.FileManager;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.apache.log4j.Logger;

/**
 * This class is used to process XML zip file validation routines
 *
 * @author Kevin Jin
 * version 1 (16/09/2003) -- Creation
 * version 1.3 04-Oct-2004 - log4j
 * version 1.3.1 16-Nov-2004 - manage a zip file with no entry, return false success
 * version 1.3.2 07-Dec-2004 - stop the unzipper keeping the file locked under exceptions
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class ZipFileValidator {

   private static Logger logger = Logger.getLogger(ZipFileValidator.class);

   /**
    * Constructor
    */
   public ZipFileValidator() {}

   /**
    * This method is used to initiate the XML zip file validation process.
    *
    * @param fileDir  String
    * @param fileName String
    * @return ZipFileInfo -- Information retrieved from the XML ZIP file
    */
   public ZipFileInfo validate(String fileDir, String fileName) {

      ZipFileInfo zpdw = new ZipFileInfo();
      String zipFileName = fileDir + "/" + fileName;
      String outFileName = "";
      boolean success = true;

      try {

         FileInputStream fis = new FileInputStream(zipFileName);

         // First, we will unzip the zip file and write to a temp file
         // Note, we will assume that only one file in each .zip file
         try {

            // Open the ZIP file
            ZipInputStream in = new ZipInputStream(fis);

            // Get the first entry
            ZipEntry entry = in.getNextEntry();
            if (entry != null) {

               outFileName = fileDir + "/" + entry.getName();
               // Open the output file
               OutputStream out = new FileOutputStream(outFileName);

               // Transfer bytes from the ZIP file to the output file
               try {
                  byte[] buf = new byte[1024];
                  int len;
                  while ((len = in.read(buf)) > 0) {
                     out.write(buf, 0, len);
                  }
               } finally {
                  // Close the streams
                  out.close();
               }
            } else {

               // no first entry in the zip file
               Util_Logger.warning(logger, "Can not unzip file - missing entry in file " + zipFileName);
               success = false;

            }

            in.close();

         } catch (IOException e) {
            Util_Logger.warning(logger, "Can not unzip file " + zipFileName, e);
            success = false;
            fis.close();
         }
      } catch (Exception e) {

         Util_Logger.warning(logger, "Can not find file " + zipFileName, e);
         success = false;
      }

      if (success) {

         // Now, let's validate the temp XML file and retrieve information from it
         ZipFileParser parser = new ZipFileParser();
         parser.parser(outFileName, zpdw);
      } else {
         // The process failed
         zpdw.setProcessSuccess(false);
      }

      //at last, we should remove the unziped temp XML file
      try {
         FileManager fm = new FileManager();
         fm.deleteFile(outFileName);
      } catch (Exception e) {
         // we can not delete the temperary file
         Util_Logger.warning(logger, "Can not delete " + outFileName, e);
      }

      return zpdw;

   }

}

